@@ -58,7 +58,7 @@ def test_access_root_of_static_handler(tmp_dir_path, loop, test_client,
     with open(my_file_path, 'w') as fw:
         fw.write('world')
 
-    app = web.Application(loop=loop)
+    app = web.Application()
 
     # Register global static route:
     app.router.add_static('/', tmp_dir_path, show_index=show_index)
@@ -72,7 +72,6 @@ def test_access_root_of_static_handler(tmp_dir_path, loop, test_client,
         assert r.headers['Content-Type'] == "text/html; charset=utf-8"
         read_ = (yield from r.read())
         assert read_ == data
-    yield from r.release()
 
 
 @pytest.mark.parametrize('data', ['hello world'])
@@ -91,7 +90,7 @@ def test_follow_symlink(tmp_dir_path, loop, test_client, data):
     my_symlink_path = os.path.join(tmp_dir_path, 'my_symlink')
     os.symlink(my_dir_path, my_symlink_path)
 
-    app = web.Application(loop=loop)
+    app = web.Application()
 
     # Register global static route:
     app.router.add_static('/', tmp_dir_path, follow_symlinks=True)
@@ -102,8 +101,6 @@ def test_follow_symlink(tmp_dir_path, loop, test_client, data):
     assert r.status == 200
     assert (yield from r.text()) == data
 
-    yield from r.release()
-
 
 @pytest.mark.parametrize('dir_name,filename,data', [
     ('', 'test file.txt', 'test text'),
@@ -126,7 +123,7 @@ def test_access_to_the_file_with_spaces(tmp_dir_path, loop, test_client,
     with open(my_file_path, 'w') as fw:
         fw.write(data)
 
-    app = web.Application(loop=loop)
+    app = web.Application()
 
     url = os.path.join('/', dir_name, filename)
 
@@ -136,7 +133,6 @@ def test_access_to_the_file_with_spaces(tmp_dir_path, loop, test_client,
     r = yield from client.get(url)
     assert r.status == 200
     assert (yield from r.text()) == data
-    yield from r.release()
 
 
 @asyncio.coroutine
@@ -146,7 +142,7 @@ def test_access_non_existing_resource(tmp_dir_path, loop, test_client):
     Try to access a non-exiting resource and make sure that 404 HTTP status
     returned.
     """
-    app = web.Application(loop=loop)
+    app = web.Application()
 
     # Register global static route:
     app.router.add_static('/', tmp_dir_path, show_index=True)
@@ -155,7 +151,6 @@ def test_access_non_existing_resource(tmp_dir_path, loop, test_client):
     # Request the root of the static directory.
     r = yield from client.get('/non_existing_resource')
     assert r.status == 404
-    yield from r.release()
 
 
 @pytest.mark.parametrize('registered_path,request_url', [
@@ -168,7 +163,7 @@ def test_url_escaping(loop, test_client, registered_path, request_url):
     """
     Tests accessing a resource with
     """
-    app = web.Application(loop=loop)
+    app = web.Application()
 
     def handler(_):
         return web.Response()
@@ -177,7 +172,6 @@ def test_url_escaping(loop, test_client, registered_path, request_url):
 
     r = yield from client.get(request_url)
     assert r.status == 200
-    yield from r.release()
 
 
 @asyncio.coroutine
@@ -190,7 +184,7 @@ def test_unauthorized_folder_access(tmp_dir_path, loop, test_client):
     my_dir_path = os.path.join(tmp_dir_path, 'my_dir')
     os.mkdir(my_dir_path)
 
-    app = web.Application(loop=loop)
+    app = web.Application()
 
     with mock.patch('pathlib.Path.__new__') as path_constructor:
         path = MagicMock()
@@ -207,8 +201,6 @@ def test_unauthorized_folder_access(tmp_dir_path, loop, test_client):
         r = yield from client.get('/my_dir')
         assert r.status == 403
 
-    yield from r.release()
-
 
 @asyncio.coroutine
 def test_access_symlink_loop(tmp_dir_path, loop, test_client):
@@ -218,7 +210,7 @@ def test_access_symlink_loop(tmp_dir_path, loop, test_client):
     my_dir_path = os.path.join(tmp_dir_path, 'my_symlink')
     os.symlink(my_dir_path, my_dir_path)
 
-    app = web.Application(loop=loop)
+    app = web.Application()
 
     # Register global static route:
     app.router.add_static('/', tmp_dir_path, show_index=True)
@@ -228,8 +220,6 @@ def test_access_symlink_loop(tmp_dir_path, loop, test_client):
     r = yield from client.get('/my_symlink')
     assert r.status == 404
 
-    yield from r.release()
-
 
 @asyncio.coroutine
 def test_access_special_resource(tmp_dir_path, loop, test_client):
@@ -238,7 +228,7 @@ def test_access_special_resource(tmp_dir_path, loop, test_client):
     Checks that if a special resource is accessed (f.e. named pipe or UNIX
     domain socket) then 404 HTTP status returned.
     """
-    app = web.Application(loop=loop)
+    app = web.Application()
 
     with mock.patch('pathlib.Path.__new__') as path_constructor:
         special = MagicMock()
@@ -261,12 +251,10 @@ def test_access_special_resource(tmp_dir_path, loop, test_client):
         r = yield from client.get('/special')
         assert r.status == 404
 
-    yield from r.release()
-
 
 @asyncio.coroutine
 def test_partialy_applied_handler(loop, test_client):
-    app = web.Application(loop=loop)
+    app = web.Application()
 
     @asyncio.coroutine
     def handler(data, request):
@@ -278,7 +266,6 @@ def test_partialy_applied_handler(loop, test_client):
     r = yield from client.get('/')
     data = (yield from r.read())
     assert data == b'hello'
-    yield from r.release()
 
 
 def test_system_route():
@@ -310,3 +297,33 @@ def test_412_is_returned(loop, test_client):
     resp = yield from client.get('/')
 
     assert resp.status == 412
+
+
+@asyncio.coroutine
+def test_allow_head(loop, test_client):
+    """
+    Test allow_head on routes.
+    """
+    app = web.Application()
+
+    def handler(_):
+        return web.Response()
+    app.router.add_get('/a', handler, name='a')
+    app.router.add_get('/b', handler, allow_head=False, name='b')
+    client = yield from test_client(app)
+
+    r = yield from client.get('/a')
+    assert r.status == 200
+    yield from r.release()
+
+    r = yield from client.head('/a')
+    assert r.status == 200
+    yield from r.release()
+
+    r = yield from client.get('/b')
+    assert r.status == 200
+    yield from r.release()
+
+    r = yield from client.head('/b')
+    assert r.status == 405
+    yield from r.release()
