@@ -2,13 +2,13 @@ import asyncio
 from unittest import mock
 
 import pytest
+from multidict import CIMultiDict
 
-from aiohttp import (CIMultiDict, WSMessage, WSMsgType, errors, helpers,
-                     signals, web)
+from aiohttp import WSMessage, WSMsgType, helpers, signals
 from aiohttp.log import ws_logger
 from aiohttp.test_utils import make_mocked_coro, make_mocked_request
 from aiohttp.web import HTTPBadRequest, HTTPMethodNotAllowed, WebSocketResponse
-from aiohttp.web_ws import WebSocketReady
+from aiohttp.web_ws import WS_CLOSED_MESSAGE, WebSocketReady
 
 
 @pytest.fixture
@@ -22,18 +22,21 @@ def app(loop):
 
 @pytest.fixture
 def writer():
-    return mock.Mock()
+    writer = mock.Mock()
+    writer.drain.return_value = ()
+    writer.write_eof.return_value = ()
+    return writer
 
 
 @pytest.fixture
-def reader():
+def protocol():
     ret = mock.Mock()
     ret.set_parser.return_value = ret
     return ret
 
 
 @pytest.fixture
-def make_request(app, writer, reader):
+def make_request(app, protocol, writer):
     def maker(method, path, headers=None, protocols=False):
         if headers is None:
             headers = CIMultiDict(
@@ -46,8 +49,9 @@ def make_request(app, writer, reader):
         if protocols:
             headers['SEC-WEBSOCKET-PROTOCOL'] = 'chat, superchat'
 
-        return make_mocked_request(method, path, headers,
-                                   app=app, writer=writer, reader=reader)
+        return make_mocked_request(
+            method, path, headers,
+            app=app, protocol=protocol, payload_writer=writer)
 
     return maker
 
@@ -253,6 +257,7 @@ def test_send_str_closed(make_request, mocker):
     req = make_request('GET', '/')
     ws = WebSocketResponse()
     yield from ws.prepare(req)
+    ws._reader.feed_data(WS_CLOSED_MESSAGE, 0)
     yield from ws.close()
 
     mocker.spy(ws_logger, 'warning')
@@ -265,6 +270,7 @@ def test_send_bytes_closed(make_request, mocker):
     req = make_request('GET', '/')
     ws = WebSocketResponse()
     yield from ws.prepare(req)
+    ws._reader.feed_data(WS_CLOSED_MESSAGE, 0)
     yield from ws.close()
 
     mocker.spy(ws_logger, 'warning')
@@ -277,6 +283,7 @@ def test_send_json_closed(make_request, mocker):
     req = make_request('GET', '/')
     ws = WebSocketResponse()
     yield from ws.prepare(req)
+    ws._reader.feed_data(WS_CLOSED_MESSAGE, 0)
     yield from ws.close()
 
     mocker.spy(ws_logger, 'warning')
@@ -289,6 +296,7 @@ def test_ping_closed(make_request, mocker):
     req = make_request('GET', '/')
     ws = WebSocketResponse()
     yield from ws.prepare(req)
+    ws._reader.feed_data(WS_CLOSED_MESSAGE, 0)
     yield from ws.close()
 
     mocker.spy(ws_logger, 'warning')
@@ -301,6 +309,7 @@ def test_pong_closed(make_request, mocker):
     req = make_request('GET', '/')
     ws = WebSocketResponse()
     yield from ws.prepare(req)
+    ws._reader.feed_data(WS_CLOSED_MESSAGE, 0)
     yield from ws.close()
 
     mocker.spy(ws_logger, 'warning')
@@ -313,13 +322,14 @@ def test_close_idempotent(make_request, writer):
     req = make_request('GET', '/')
     ws = WebSocketResponse()
     yield from ws.prepare(req)
+    ws._reader.feed_data(WS_CLOSED_MESSAGE, 0)
     assert (yield from ws.close(code=1, message='message1'))
     assert ws.closed
     assert not (yield from ws.close(code=2, message='message2'))
 
 
 @asyncio.coroutine
-def test_start_invalid_method(make_request):
+def test_prepare_invalid_method(make_request):
     req = make_request('POST', '/')
     ws = WebSocketResponse()
     with pytest.raises(HTTPMethodNotAllowed):
@@ -327,7 +337,7 @@ def test_start_invalid_method(make_request):
 
 
 @asyncio.coroutine
-def test_start_without_upgrade(make_request):
+def test_prepare_without_upgrade(make_request):
     req = make_request('GET', '/',
                        headers=CIMultiDict({}))
     ws = WebSocketResponse()
@@ -344,7 +354,6 @@ def test_wait_closed_before_start():
 
 @asyncio.coroutine
 def test_write_eof_not_started():
-
     ws = WebSocketResponse()
     with pytest.raises(RuntimeError):
         yield from ws.write_eof()
@@ -355,6 +364,7 @@ def test_write_eof_idempotent(make_request):
     req = make_request('GET', '/')
     ws = WebSocketResponse()
     yield from ws.prepare(req)
+    ws._reader.feed_data(WS_CLOSED_MESSAGE, 0)
     yield from ws.close()
 
     yield from ws.write_eof()
@@ -363,17 +373,19 @@ def test_write_eof_idempotent(make_request):
 
 
 @asyncio.coroutine
-def test_receive_exc_in_reader(make_request, loop, reader):
+def test_receive_exc_in_reader(make_request, loop):
     req = make_request('GET', '/')
     ws = WebSocketResponse()
     yield from ws.prepare(req)
 
+    ws._reader = mock.Mock()
     exc = ValueError()
     res = helpers.create_future(loop)
     res.set_exception(exc)
-    reader.read = make_mocked_coro(res)
-    ws._resp_impl.transport.drain.return_value = helpers.create_future(loop)
-    ws._resp_impl.transport.drain.return_value.set_result(True)
+    ws._reader.read = make_mocked_coro(res)
+    ws._payload_writer.drain = mock.Mock()
+    ws._payload_writer.drain.return_value = helpers.create_future(loop)
+    ws._payload_writer.drain.return_value.set_result(True)
 
     msg = yield from ws.receive()
     assert msg.type == WSMsgType.ERROR
@@ -383,57 +395,41 @@ def test_receive_exc_in_reader(make_request, loop, reader):
 
 
 @asyncio.coroutine
-def test_receive_cancelled(make_request, loop, reader):
+def test_receive_cancelled(make_request, loop):
     req = make_request('GET', '/')
     ws = WebSocketResponse()
     yield from ws.prepare(req)
 
+    ws._reader = mock.Mock()
     res = helpers.create_future(loop)
     res.set_exception(asyncio.CancelledError())
-    reader.read = make_mocked_coro(res)
+    ws._reader.read = make_mocked_coro(res)
 
     with pytest.raises(asyncio.CancelledError):
         yield from ws.receive()
 
 
 @asyncio.coroutine
-def test_receive_timeouterror(make_request, loop, reader):
+def test_receive_timeouterror(make_request, loop):
     req = make_request('GET', '/')
     ws = WebSocketResponse()
     yield from ws.prepare(req)
 
+    ws._reader = mock.Mock()
     res = helpers.create_future(loop)
     res.set_exception(asyncio.TimeoutError())
-    reader.read = make_mocked_coro(res)
+    ws._reader.read = make_mocked_coro(res)
 
     with pytest.raises(asyncio.TimeoutError):
         yield from ws.receive()
 
 
-@asyncio.coroutine
-def test_receive_client_disconnected(make_request, loop, reader):
-    req = make_request('GET', '/')
-    ws = WebSocketResponse()
-    yield from ws.prepare(req)
-
-    exc = errors.ClientDisconnectedError()
-    res = helpers.create_future(loop)
-    res.set_exception(exc)
-    reader.read = make_mocked_coro(res)
-
-    msg = yield from ws.receive()
-    assert ws.closed
-    assert msg.type == WSMsgType.CLOSED
-    assert msg.type is msg.tp
-    assert msg.data is None
-    assert ws.exception() is None
-
-
 @asyncio.coroutine
 def test_multiple_receive_on_close_connection(make_request):
     req = make_request('GET', '/')
     ws = WebSocketResponse()
     yield from ws.prepare(req)
+    ws._reader.feed_data(WS_CLOSED_MESSAGE, 0)
     yield from ws.close()
 
     yield from ws.receive()
@@ -457,25 +453,27 @@ def test_concurrent_receive(make_request):
 
 
 @asyncio.coroutine
-def test_close_exc(make_request, reader, loop, mocker):
+def test_close_exc(make_request, loop, mocker):
     req = make_request('GET', '/')
 
     ws = WebSocketResponse()
     yield from ws.prepare(req)
 
+    ws._reader = mock.Mock()
     exc = ValueError()
-    reader.read.return_value = helpers.create_future(loop)
-    reader.read.return_value.set_exception(exc)
-    ws._resp_impl.transport.drain.return_value = helpers.create_future(loop)
-    ws._resp_impl.transport.drain.return_value.set_result(True)
+    ws._reader.read.return_value = helpers.create_future(loop)
+    ws._reader.read.return_value.set_exception(exc)
+    ws._payload_writer.drain = mock.Mock()
+    ws._payload_writer.drain.return_value = helpers.create_future(loop)
+    ws._payload_writer.drain.return_value.set_result(True)
 
     yield from ws.close()
     assert ws.closed
     assert ws.exception() is exc
 
     ws._closed = False
-    reader.read.return_value = helpers.create_future(loop)
-    reader.read.return_value.set_exception(asyncio.CancelledError())
+    ws._reader.read.return_value = helpers.create_future(loop)
+    ws._reader.read.return_value.set_exception(asyncio.CancelledError())
     with pytest.raises(asyncio.CancelledError):
         yield from ws.close()
     assert ws.close_code == 1006
@@ -491,7 +489,6 @@ def test_close_exc2(make_request):
     exc = ValueError()
     ws._writer = mock.Mock()
     ws._writer.close.side_effect = exc
-
     yield from ws.close()
     assert ws.closed
     assert ws.exception() is exc
@@ -503,22 +500,10 @@ def test_close_exc2(make_request):
 
 
 @asyncio.coroutine
-def test_start_twice_idempotent(make_request):
+def test_prepare_twice_idempotent(make_request):
     req = make_request('GET', '/')
     ws = WebSocketResponse()
-    with pytest.warns(DeprecationWarning):
-        impl1 = yield from ws.start(req)
-        impl2 = yield from ws.start(req)
-        assert impl1 is impl2
 
-
-def test_can_start_ok(make_request):
-    req = make_request('GET', '/', protocols=True)
-    ws = WebSocketResponse(protocols=('chat',))
-    with pytest.warns(DeprecationWarning):
-        assert (True, 'chat') == ws.can_start(req)
-
-
-def test_msgtype_alias():
-    # deprecated since 1.0
-    assert web.MsgType is WSMsgType
+    impl1 = yield from ws.prepare(req)
+    impl2 = yield from ws.prepare(req)
+    assert impl1 is impl2
