@@ -26,12 +26,16 @@ ACCEPTABLE_LOG_FORMAT = '%a "%{Referrer}i" %s'
 class BaseTestWorker:
 
     def __init__(self):
-        self.pid = 'pid'
         self.servers = {}
         self.exit_code = 0
         self.cfg = mock.Mock()
         self.cfg.graceful_timeout = 100
 
+        try:
+            self.pid = 'pid'
+        except:
+            pass
+
 
 class AsyncioWorker(BaseTestWorker, base_worker.GunicornWebWorker):
     pass
@@ -159,10 +163,8 @@ def test_make_handler_wsgi(worker, mocker):
     worker.cfg.access_log_format = ACCEPTABLE_LOG_FORMAT
     mocker.spy(worker, '_get_valid_log_format')
 
-    f = worker.make_handler(worker.wsgi)
-    assert isinstance(f, base_worker.WSGIServer)
-    assert isinstance(f(), base_worker.WSGIServerHttpProtocol)
-    assert worker._get_valid_log_format.called
+    with pytest.raises(RuntimeError):
+        worker.make_handler(worker.wsgi)
 
 
 @pytest.mark.parametrize('source,result', [
